@@ -1,6 +1,6 @@
 package Perinci::Sub::Gen::AccessTable;
 
-use 5.010;
+use 5.010001;
 use Log::Any '$log';
 use strict;
 use warnings;
@@ -19,7 +19,7 @@ require Exporter;
 our @ISA       = qw(Exporter);
 our @EXPORT_OK = qw(gen_read_table_func);
 
-our $VERSION = '0.17'; # VERSION
+our $VERSION = '0.18'; # VERSION
 
 our %SPEC;
 
@@ -590,6 +590,7 @@ sub _gen_func {
     my $func_args = $func_meta->{args};
     my $func = sub {
         my %args = @_;
+        my $hooks = $opts->{hooks};
 
         # XXX schema
         while (my ($ak, $av) = each %$func_args) {
@@ -603,13 +604,22 @@ sub _gen_func {
             }
         }
 
+        for ('before_parse_query') {
+            $hooks->{$_}->(%args, _stage=>$_) if $hooks->{$_};
+        }
         my $res = __parse_query($table_spec, $opts, $func_meta, \%args);
+        for ('after_parse_query') {
+            $hooks->{$_}->(%args, _stage=>$_, _parse_res=>$res) if $hooks->{$_};
+        }
         return $res unless $res->[0] == 200;
         my $query = $res->[2];
 
         # retrieve data
         my $data;
         my $metadata = {};
+        for ('before_fetch_data') {
+            $hooks->{$_}->(%args, _stage=>$_, _query=>$query) if $hooks->{$_};
+        }
         if (__is_aoa($table_data) || __is_aoh($table_data)) {
             $data = $table_data;
         } elsif (reftype($table_data) eq 'CODE') {
@@ -630,6 +640,10 @@ sub _gen_func {
             # this should be impossible, already checked earlier
             die "BUG: Data is not an array";
         }
+        for ('after_fetch_data') {
+            $hooks->{$_}->(%args, _stage=>$_, _query=>$query, _data=>$data)
+                if $hooks->{$_};
+        }
 
         # this will be the final result.
         my @r;
@@ -1048,6 +1062,19 @@ $opts) where $v is the filter value (from the function argument) and $r the
 hashref record value. $opts is currently empty. Code should return true if
 record satisfies the filter.
 
+_
+        },
+        hooks => {
+            schema      => [hash => {of=>'code*'}],
+            summary     => 'Supply hooks',
+            description => <<'_',
+
+You can instruct the generated function to execute codes in various stages by
+using hooks. Currently available hooks are: `before_parse_query`,
+`after_parse_query`, `before_fetch_data`, `after_fetch_data`. Hooks will be
+passed the function arguments as well as zero or more additional ones,
+including: `_stage` (name of stage).
+
 _
         },
     },
@@ -1126,6 +1153,7 @@ sub _gen_read_table_func {
         case_insensitive_search    => $args{case_insensitive_search} // 1,
         (map { ("default_$_" => $dav->{$_}) } keys %$dav),
         custom_filters             => $cff,
+        hooks                      => $args{hooks} // {},
     };
 
     my $res;
@@ -1157,13 +1185,15 @@ sub _gen_read_table_func {
 __END__
 =pod
 
+=encoding utf-8
+
 =head1 NAME
 
 Perinci::Sub::Gen::AccessTable - Generate function (and its Rinci metadata) to access table data
 
 =head1 VERSION
 
-version 0.17
+version 0.18
 
 =head1 SYNOPSIS
 
@@ -1281,9 +1311,22 @@ L<Rinci>
 
 L<Perinci::CmdLine>
 
+=head1 AUTHOR
+
+Steven Haryanto <stevenharyanto@gmail.com>
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 2013 by Steven Haryanto.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
 =head1 FUNCTIONS
 
 
+None are exported by default, but they are exportable.
+
 =head2 gen_read_table_func(%args) -> [status, msg, result, meta]
 
 Generate function (and its metadata) to read table data.
@@ -1300,7 +1343,7 @@ arguments.
 
 =item *
 
-B<with_field_names> => BOOL (default 1)
+I<with_field_names> => BOOL (default 1)
 
   If set to 1, function will return records of field values along with field
   names (hashref), e.g. {id=>'ID', country=>'Indonesia', capital=>'Jakarta'}. If
@@ -1311,49 +1354,49 @@ B<with_field_names> => BOOL (default 1)
 
 =item *
 
-B<detail> => BOOL (default 0)
+I<detail> => BOOL (default 0)
 
   This is a field selection option. If set to 0, function will return PK field
   only. If this argument is set to 1, then all fields will be returned (see also
-  B<fields> to instruct function to return some fields only).
+  I<fields> to instruct function to return some fields only).
 
 
 
 =item *
 
-B<fields> => ARRAY
+I<fields> => ARRAY
 
   This is a field selection option. If you only want certain fields, specify
-  them here (see also B<detail>).
+  them here (see also I<detail>).
 
 
 
 =item *
 
-B<result_limit> => INT (default undef)
+I<result_limit> => INT (default undef)
 
 
 
 =item *
 
-B<result_start> => INT (default 1)
+I<result_start> => INT (default 1)
 
-  The B<result_limit> and B<result_start> arguments are paging options, they work
+  The I<result_limit> and I<result_start> arguments are paging options, they work
   like LIMIT clause in SQL, except that index starts at 1 and not 0. For
-  example, to return the first 20 records in the result, set B<result_limit> to
+  example, to return the first 20 records in the result, set I<result_limit> to
 
 
 
 =item *
 
-To return the next 20 records, set B<result_limit> to 20 and B<result_start>
+To return the next 20 records, set I<result_limit> to 20 and I<result_start>
   to 21.
 
 
 
 =item *
 
-B<random> => BOOL (default 0)
+I<random> => BOOL (default 0)
 
   The random argument is an ordering option. If set to true, order of records
   returned will be shuffled first. This happened before paging.
@@ -1362,7 +1405,7 @@ B<random> => BOOL (default 0)
 
 =item *
 
-B<sort> => STR
+I<sort> => STR
 
   The sort argument is an ordering option, containing name of field. A - prefix
   signifies descending instead of ascending order. Multiple fields are allowed,
@@ -1372,12 +1415,12 @@ B<sort> => STR
 
 =item *
 
-B<q> => STR
+I<q> => STR
 
   A filtering option. By default, all fields except those specified with
   searchable=0 will be searched using simple case-insensitive string search.
   There are a few options to customize this, using these gen arguments:
-  B<word_search>, B<case_insensitive_search>, and B<custom_search>.
+  I<word_search>, I<case_insensitive_search>, and I<custom_search>.
 
 
 
@@ -1394,23 +1437,23 @@ Filter arguments
 
 =item *
 
-B<FIELD.is> and B<FIELD.isnt> arguments for each field. Only records with
+I<FIELD.is> and I<FIELD.isnt> arguments for each field. Only records with
  field equalling (or not equalling) value exactly ('==' or 'eq') will be
- included. If doesn't clash with other function arguments, B<FIELD> will also
- be added as an alias for B<FIELD.is>.
+ included. If doesn't clash with other function arguments, I<FIELD> will also
+ be added as an alias for I<FIELD.is>.
 
 
 
 =item *
 
-B<FIELD.has> and B<FIELD.lacks> array arguments for each set field. Only
+I<FIELD.has> and I<FIELD.lacks> array arguments for each set field. Only
 records with field having or lacking certain value will be included.
 
 
 
 =item *
 
-B<FIELD.min> and B<FIELD.max> for each int/float/str field. Only records with
+I<FIELD.min> and I<FIELD.max> for each int/float/str field. Only records with
 field greater/equal than, or less/equal than a certain value will be
 included.
 
@@ -1418,7 +1461,7 @@ included.
 
 =item *
 
-B<FIELD.contains> and B<FIELD.not_contains> for each str field. Only records
+I<FIELD.contains> and I<FIELD.not_contains> for each str field. Only records
 with field containing (or not containing) certain value (substring) will be
 included.
 
@@ -1426,7 +1469,7 @@ included.
 
 =item *
 
-B<FIELD.matches> and B<FIELD.not_matches> for each str field. Only records
+I<FIELD.matches> and I<FIELD.not_matches> for each str field. Only records
 with field matching (or not matching) certain value (regex) (or will be
 included. Function will return 400 if regex is invalid. These arguments will
 not be generated if 'filterable_regex' clause in field specification is set
@@ -1450,8 +1493,8 @@ Supply custom filters.
 
 A hash of filter name and definitions. Filter name will be used as generated
 function's argument and must not clash with other arguments. Filter definition
-is a hash containing these keys: B<meta> (hash, argument metadata), B<code>,
-B<fields> (array, list of table fields related to this field).
+is a hash containing these keys: I<meta> (hash, argument metadata), I<code>,
+I<fields> (array, list of table fields related to this field).
 
 Code will be called for each record to be filtered and will be supplied ($r, $v,
 $opts) where $v is the filter value (from the function argument) and $r the
@@ -1499,7 +1542,7 @@ Supply default 'sort' value in generated function's metadata.
 
 Supply default 'with_field_names' value in generated function's metadata.
 
-=item * B<description>* => I<str>
+=item * B<description> => I<str>
 
 Generated function's description.
 
@@ -1507,6 +1550,16 @@ Generated function's description.
 
 Decide whether generated function will support searching (argument q).
 
+=item * B<hooks> => I<hash>
+
+Supply hooks.
+
+You can instruct the generated function to execute codes in various stages by
+using hooks. Currently available hooks are: C<before_parse_query>,
+C<after_parse_query>, C<before_fetch_data>, C<after_fetch_data>. Hooks will be
+passed the function arguments as well as zero or more additional ones,
+including: C<_stage> (name of stage).
+
 =item * B<install> => I<bool> (default: 1)
 
 Whether to install generated function (and metadata).
@@ -1520,7 +1573,7 @@ false to skip installing.
 Choose language for function metadata.
 
 This function can generate metadata containing text from one or more languages.
-For example if you set 'langs' to ['enB<US', 'id>ID'] then the generated function
+For example if you set 'langs' to ['enI<US', 'id>ID'] then the generated function
 metadata might look something like this:
 
     {
@@ -1540,7 +1593,7 @@ metadata might look something like this:
 
 Generated function's name, e.g. `myfunc`.
 
-=item * B<package>* => I<str>
+=item * B<package> => I<str>
 
 Generated function's package, e.g. `My::Package`.
 
@@ -1549,7 +1602,7 @@ supply this if you set C<install> to false.
 
 If not specified, caller's package will be used by default.
 
-=item * B<summary>* => I<str>
+=item * B<summary> => I<str>
 
 Generated function's summary.
 
@@ -1573,10 +1626,10 @@ efforts.
 
 '$query' is a hashref which contains information about the query, e.g. 'args'
 (the original arguments passed to the generated function, e.g. {random=>1,
-resultB<limit=>1, field1>match=>'f.+'}), 'mentionedB<fields' which lists fields
+resultI<limit=>1, field1>match=>'f.+'}), 'mentionedI<fields' which lists fields
 that are mentioned in either filtering arguments or fields or ordering,
 'requested>fields' (fields mentioned in list of fields to be returned),
-'sortB<fields' (fields mentioned in sort arguments), 'filter>fields' (fields
+'sortI<fields' (fields mentioned in sort arguments), 'filter>fields' (fields
 mentioned in filter arguments).
 
 =item * B<table_spec>* => I<hash>
@@ -1599,7 +1652,7 @@ true).
 Decide whether generated function will perform word searching instead of string searching.
 
 For example, if search term is 'pine' and field value is 'green pineapple',
-search will match if wordB<search=false, but won't match under word>search.
+search will match if wordI<search=false, but won't match under word>search.
 
 This will not have effect under 'custom_search'.
 
@@ -1609,16 +1662,5 @@ Return value:
 
 Returns an enveloped result (an array). First element (status) is an integer containing HTTP status code (200 means OK, 4xx caller error, 5xx function error). Second element (msg) is a string containing error message, or 'OK' if status is 200. Third element (result) is optional, the actual result. Fourth element (meta) is called result metadata and is optional, a hash that contains extra information.
 
-=head1 AUTHOR
-
-Steven Haryanto <stevenharyanto@gmail.com>
-
-=head1 COPYRIGHT AND LICENSE
-
-This software is copyright (c) 2012 by Steven Haryanto.
-
-This is free software; you can redistribute it and/or modify it under
-the same terms as the Perl 5 programming language system itself.
-
 =cut
 
